@@ -94,7 +94,8 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	combo->addItem("Русский");
 	combo->addItem("Slovenščina");
 	combo->addItem("Svenska");
-	combo->addItem("汉语");
+	combo->addItem("简体中文");
+	combo->addItem("繁體中文");
 	QString lang = settings->custom_lang.left(2);
 	if(lang == "ca") combo->setCurrentIndex(1);
 	else if(lang == "de") combo->setCurrentIndex(2);
@@ -108,7 +109,8 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	else if(lang == "ru") combo->setCurrentIndex(10);
 	else if(lang == "sl") combo->setCurrentIndex(11);
 	else if(lang == "sv") combo->setCurrentIndex(12);
-	else if(lang == "zh") combo->setCurrentIndex(13);
+	else if(settings->custom_lang == "zh_CN") combo->setCurrentIndex(13);
+	else if(settings->custom_lang == "zh_TW") combo->setCurrentIndex(14);
 	else combo->setCurrentIndex(0);
 	combo->setEnabled(!settings->ignore_locale);
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(langChanged(int)));
@@ -117,7 +119,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	BOX(tr("Clear history on exit"), settings->clear_history_on_exit, clearHistoryToggled(bool));
 	l->addWidget(new QLabel(tr("Max history lines saved:"), this), r, 0); QSpinBox *spin = new QSpinBox(this); spin->setRange(0, 100000); spin->setValue(settings->max_history_lines); connect(spin, SIGNAL(valueChanged(int)), this, SLOT(maxHistoryLinesChanged(int))); l->addWidget(spin, r, 1); r++;
 	BOX(tr("Close application with Escape key"), settings->close_with_esc, closeWithEscToggled(bool));
-	BOX(tr("Use keyboard keys for RPN"), settings->rpn_keys, rpnKeysToggled(bool));
+	BOX(tr("Use keyboard keys for RPN"), settings->rpn_keys && settings->evalops.parse_options.parsing_mode != PARSING_MODE_RPN, rpnKeysToggled(bool)); rpnKeysBox = box; rpnKeysBox->setEnabled(settings->evalops.parse_options.parsing_mode != PARSING_MODE_RPN);
 	BOX(tr("Use caret for bitwise XOR"), settings->caret_as_xor, caretAsXorToggled(bool));
 	BOX(tr("Keep above other windows"), settings->always_on_top, keepAboveToggled(bool));
 	QCheckBox *box2 = new QCheckBox(tr("Preserve history height"), this); l->addWidget(box2, r, 0, 1, 2); connect(box2, SIGNAL(stateChanged(int)), this, SLOT(preserveHeightChanged(int))); r++;
@@ -214,6 +216,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	calculateDelayWidget->setEnabled(settings->status_in_history);
 	connect(calculateDelayWidget, SIGNAL(valueChanged(int)), this, SLOT(calculateDelayChanged(int)));
 	l->addWidget(calculateDelayWidget, r, 1); r++;
+	BOX(tr("Automatically calculate selection"), settings->autocalc_selection, autocalcSelectionToggled(bool));
 	l->addWidget(new QLabel(tr("Expression in history:"), this), r, 0);
 	combo = new QComboBox(this);
 	combo->addItem(tr("Parsed"), 0);
@@ -293,10 +296,17 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	decimalCommaBox = box;
 	BOX(tr("Ignore comma in numbers"), settings->evalops.parse_options.comma_as_separator, ignoreCommaToggled(bool)); ignoreCommaBox = box;
 	BOX(tr("Ignore dots in numbers"), settings->evalops.parse_options.dot_as_separator, ignoreDotToggled(bool)); ignoreDotBox = box;
-	BOX(tr("Indicate repeating decimals"), settings->printops.indicate_infinite_series, repeatingDecimalsToggled(bool));
 	if(CALCULATOR->getDecimalPoint() == COMMA) ignoreCommaBox->hide();
 	if(CALCULATOR->getDecimalPoint() == DOT) ignoreDotBox->hide();
 	BOX(tr("Copy unformatted ASCII by default"), settings->copy_ascii, copyAsciiToggled(bool));
+	l->addWidget(new QLabel(tr("Indicate repeating decimals:"), this), r, 0);
+	combo = new QComboBox(this);
+	combo->addItem(tr("Off"), REPEATING_DECIMALS_OFF);
+	combo->addItem(tr("Ellipsis"), REPEATING_DECIMALS_ELLIPSIS);
+	combo->addItem(tr("Overline"), REPEATING_DECIMALS_OVERLINE);
+	combo->setCurrentIndex(combo->findData(settings->printops.indicate_infinite_series));
+	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(repeatingDecimalsChanged(int)));
+	l->addWidget(combo, r, 1); r++;
 	l->addWidget(new QLabel(tr("Digit grouping:"), this), r, 0);
 	combo = new QComboBox(this);
 	combo->addItem(tr("None"), DIGIT_GROUPING_NONE);
@@ -448,6 +458,7 @@ void PreferencesDialog::langChanged(int i) {
 		case 11: {settings->custom_lang = "sl_SL"; break;}
 		case 12: {settings->custom_lang = "sv_SE"; break;}
 		case 13: {settings->custom_lang = "zh_CN"; break;}
+		case 14: {settings->custom_lang = "zh_TW"; break;}
 	}
 	if(!settings->custom_lang.isEmpty()) {
 		ignoreLocaleBox->setChecked(false);
@@ -494,6 +505,9 @@ void PreferencesDialog::statusDelayChanged(int v) {
 void PreferencesDialog::calculateDelayChanged(int v) {
 	settings->auto_calculate_delay = v;
 }
+void PreferencesDialog::autocalcSelectionToggled(bool b) {
+	settings->autocalc_selection = b;
+}
 void PreferencesDialog::binTwosToggled(bool b) {
 	settings->printops.twos_complement = b;
 	emit resultDisplayUpdated();
@@ -606,6 +620,10 @@ void PreferencesDialog::formatToggled(bool b) {
 void PreferencesDialog::parsingModeChanged(int i) {
 	settings->evalops.parse_options.parsing_mode = (ParsingMode) qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
 	if(settings->evalops.parse_options.parsing_mode == PARSING_MODE_CONVENTIONAL || settings->evalops.parse_options.parsing_mode == PARSING_MODE_IMPLICIT_MULTIPLICATION_FIRST) settings->implicit_question_asked = true;
+	rpnKeysBox->setEnabled(settings->evalops.parse_options.parsing_mode != PARSING_MODE_RPN);
+	rpnKeysBox->blockSignals(true);
+	rpnKeysBox->setChecked(settings->rpn_keys && settings->evalops.parse_options.parsing_mode != PARSING_MODE_RPN);
+	rpnKeysBox->blockSignals(false);
 	emit expressionFormatUpdated(false);
 }
 void PreferencesDialog::temperatureCalculationChanged(int i) {
@@ -675,9 +693,9 @@ void PreferencesDialog::roundingChanged(int i) {
 	settings->printops.rounding = (RoundingMode) qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
 	emit resultFormatUpdated();
 }
-void PreferencesDialog::repeatingDecimalsToggled(bool b) {
-	settings->printops.indicate_infinite_series = b;
-	emit resultFormatUpdated();
+void PreferencesDialog::repeatingDecimalsChanged(int i) {
+	settings->printops.indicate_infinite_series = qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
+	emit resultDisplayUpdated();
 }
 void PreferencesDialog::copyAsciiToggled(bool b) {
 	settings->copy_ascii = b;
